comment *


  This file contains routines and functions common to all flash programmers.


    ParmRecord2EAX:  This routine fetches the parameter record through ZTAG
    interface.


    ZTRead_RLE:  This routine performs ZTRead together with RLE expanding, if
    payload is RLE encoded (code "F0")


    Subroutine JEDEC_Cmd_8:  Execute flash device command in AM29F016 fashion
    (JEDEC) when device is connected with 8-bit interface



    revision history
    ----------------
    CRC 12-26-00        updated comments, added equates

end comment *

; -------------------------------------
; Get parameter record from ZTAG to EAX
; -------------------------------------
;
; This routine fetches the parameter record through ZTAG interface.
;
; Entry: none
;
; Exit: EAX - parameter value if NC
;       CY if no parameter record found
;
; Uses: BUR checksum area
;

ParmRecord2EAX  proc
                push    ebx
                push    ds
                push    ecx
                push    esi

                call    ZTPrepareRead           ; prepare for accelerated read

                ; Fetch header now

                mov     cx, 4
GetHeader:
                shl     eax, 8
                call    ZTRead
                loop    GetHeader

                cmp     eax, 7FF055FEh          ; is that a parameter?
                jz      @f

                stc                             ; show error
                jmp     ExitParmRecord
@@:
                ; Skip date and time

                mov     cx, 6
@@:
                call    ZTRead
                loop    @b

                ; Skip description

                call    ZTread
                mov     cl, al
                xor     ch, ch
@@:
                call    ZTRead
                loop    @b

                ; Skip version and length

                mov     cx, 6
@@:
                call    ZTRead
                loop    @b

                ; Get parameter and check checksum

                call    ResetCRC
                mov     cx, 4
@@:
                shl     eax, 8
                call    ZTRead
                int     17h
                loop    @b

                mov     esi, eax                ; save parameter value

                ; Get original checksum

                call    ZTRead
                shl     ax, 8
                call    ZTRead
                xchg    al, ah

                call    DSBX2Var                ; get variables block to DS:BX
                cmp     word ptr ds:[bx.YModemCRChi_C], ax
                jz      @f

                stc
                jmp     ExitParmRecord
@@:
                mov     eax, esi                ; restore parameter value
                bswap   eax                     ; bytes to dword order

ExitParmRecord:
                pop     esi
                pop     ecx
                pop     ds
                pop     ebx

                ret
ParmRecord2EAX  endp

; ---------------------
; Subroutine ZTRead_RLE
; ---------------------
;
; This routine performs ZTRead together with RLE expanding, if payload is RLE encoded (code "F0")
; It will also update the ZTAG interface read checksum with INT17, whenever the byte is
; fetched through ZTAG interface.
;
; Entry: none
;
; Exit: AL - byte read
;
; Uses: EAX
;

ZTRead_RLE      proc
                cmp     byte ptr cs:[BasketType], 0F0h
                jz      RLE

                call    ZTRead                          ; get byte when not RLE and quit
                int     17h                             ; update checksum

                jmp     Quit_ZTRead
    RLE:
                ; OK, we have RLE encoded basket. Check, if we have RLE run active already

                cmp     dword ptr cs:[RLE_Count], 0
                jnz     Run_Active

                ; No active run. Fetch the byte and check for an action.

                call    ZTRead
                int     17h

                cmp     al, 0DCh                        ; RLE "next byte" indicator?
                jnz     @f

                call    ZTRead                          ; fetch next byte ..
                int     17h

                jmp     Quit_ZTRead_RLE                 ; .. and present it "as is"
    @@:
                cmp     al, 0DDh                        ; RLE "byte start run" indicator?
                jnz     @f

                call    ZTRead                          ; fetch run byte
                int     17h

                mov     byte ptr cs:[RLE_Byte], al

                call    ZTRead                          ; fetch run count
                int     17h

                mov     dword ptr cs:[RLE_Count], 0
                mov     byte ptr cs:[RLE_Count], al
                jmp     Run_Active                      ; Do RLE now
    @@:
                cmp     al, 0DEh                        ; RLE "word start run" indicator?
                jnz     @f

                call    ZTRead                          ; fetch run byte
                int     17h

                mov     byte ptr cs:[RLE_Byte], al

                push    ax                              ; preserve AX

                call    ZTRead                          ; fetch run count low nibble
                int     17h
                shl     ax, 8
                call    ZTRead                          ; fetch run count high nibble
                int     17h

                xchg    al, ah                          ; bytes to word value

                mov     dword ptr cs:[RLE_Count], 0
                mov     word ptr cs:[RLE_Count], ax

                pop     ax

                jmp     Run_Active                      ; Do RLE now

    @@:
                cmp     al, 0DFh                        ; RLE "dword start run" indicator?
                jnz     @f

                call    ZTRead                          ; fetch run byte
                int     17h

                mov     byte ptr cs:[RLE_Byte], al

                push    eax

                call    ZTRead                          ; fetch run count low nibble
                int     17h
                shl     eax, 8
                call    ZTRead                          ; fetch run count middle1 nibble
                int     17h
                shl     eax, 8
                call    ZTRead                          ; fetch run count middle2 nibble
                int     17h
                shl     eax, 8
                call    ZTRead                          ; fetch run count high nibble
                int     17h

                bswap   eax                             ; bytes to dword order

                mov     dword ptr cs:[RLE_Count], eax

                pop     eax

                jmp     Run_Active                      ; Do RLE now
    @@:
                jmp     Quit_ZTRead_RLE                 ; no RLE indicators, present current byte "as is"
    Run_Active:
                ; We have active run. Replicate the given byte and decrement run counter

                mov     al, byte ptr cs:[RLE_Byte]
                dec     dword ptr cs:[RLE_Count]

    Quit_ZTRead_RLE:
                ; Update RLE checksumming

                push    eax                             ; preserve EAX - ZTRead definition requires it!
                and     eax, 0FFh
                xor     dword ptr cs:[RLE_Chk], eax
                rol     dword ptr cs:[RLE_Chk], 1
                pop     eax
                inc     dword ptr cs:[RLE_Expanded]
    Quit_ZTRead:
                ret
    ZTRead_RLE  endp



;   ---------------------
;   Subroutine JEDEC_Cmd_8
;   ---------------------
;
;
; Execute flash device command in AM29F016 fashion (JEDEC) when device is
; connected with 8-bit interface
;
; This function expects the PRGBASE variable set to programming window start address
;
; Entry: AL - command code
;
; Exit: none
;
; Uses: none
;

JEDEC_Cmd_8     proc
                push    ds
                push    si

                push    ax

                push    (PRGBASE/10h)
                pop     ds

                ; The commmand sequence is documented to be
                ; 1. write AAH to 555H
                ; 2. write 55H to 2AAH
                ; 3. write xxH to 555H  xx=90 DevID, A0 Program, 80 prep erase
                ;

                mov     al, 0AAh                ; CMD sequence part 1
                mov     si, 0555h
                mov     ds:[si], al

                mov     al, 055h                ; CMD sequence part 2
                mov     si, 02AAh
                mov     ds:[si], al

                pop     ax                      ; restore command code

                mov     si, 0555h               ; CMD sequence part 3
                mov     ds:[si], al

                pop     si
                pop     ds
                ret
JEDEC_Cmd_8     endp


;
; RLE expander data area (used by expansion algorithm)

RLE_Count       dd      0               ; number of bytes to replicate at run
RLE_Byte        db      0               ; byte to replicate if run is active
RLE_Chk         dd      0               ; calculated checksum of expanded data
RLE_Expanded    dd      0               ; total length of expanded data

; RLE Header data (retrieved from basket)
;

RLEHeader       label word              ; OFFSET RLEHeader is pointer to this data block
RLE_Mode        db      0               ; compression mode (not used!)
RLE_Destination dd      0               ; destination address (not used!)
RLE_UnCompLen   dd      0               ; length of uncompressed data
RLE_CompLen     dd      0               ; length of the compressed image
RLE_CheckSum    dd      0               ; checksum of uncompressed data

BasketType      db      0               ; Basket type code (FF for normal, F0 for RLE)
BasketSize      dd      ?
